import sys


BUILTIN = (
    '__future__',
    'collections',
    'datetime',
    'hashlib',
    'logging',
    'os',
    'random',
    're',
    'shutil',
    'sys',
    'tempfile',
    'unittest',
)


def get_package(module):
    try:
        return module[:module.index('.')]
    except ValueError:
        return module


def organize(content, third_party_packages=None, homemade_packages=None):
    if not third_party_packages:
        third_party_packages = list()
    if not homemade_packages:
        homemade_packages = list()

    # Strip irrelevant whitespaces from file
    no_irrelevant_whitespace = list()
    for line in content.split('\n'):
        no_irrelevant_whitespace.append(line.rstrip())
    content = '\n'.join(no_irrelevant_whitespace)

    # If file does not start with imports, it's not supported yet.
    if not (content.startswith('from') or content.startswith('import')):
        return content

    # Save not-import Python code for later
    try:
        imports_end = content.index('\n\n\n')
    except ValueError:
        python_code = ''
        content = content
    else:
        python_code = content[imports_end + 3:]
        content = content[:imports_end]

    # Remove empty lines and all starting and trailing whitespaces
    lines = list()
    for line in content.split('\n'):
        no_whitespace_line = line.strip()
        if no_whitespace_line:
            lines.append(no_whitespace_line)

    # Make multiple line imports in single line imports
    import_lines = list()
    for idx, line in enumerate(lines):
        if line.startswith('from') or line.startswith('import'):
            import_line = line
            current_idx = idx
            while import_line.endswith('\\'):
                import_line = import_line[:len(import_line) - 1]
                import_line = import_line.strip()
                import_line += ' ' + lines[current_idx + 1].strip()
                current_idx += 1
            import_lines.append(import_line)

    # Order import modules and objects
    modules = list()
    from_imports = dict()
    import_imports = list()
    for line in import_lines:
        if line.startswith('from'):
            import_module = line[5:line.index('import') - 1]
            import_objects = line[line.index('import') + 7:].split(', ')
            if import_module not in modules:
                modules.append(import_module)
            if import_module in from_imports:
                from_imports[import_module] += import_objects
            else:
                from_imports[import_module] = import_objects
        else:
            import_modules = line[7:].split(', ')
            for module in import_modules:
                if module not in modules:
                    modules.append(module)
                import_imports.append(module)

    builtin = list()
    third_party = list()
    homemade = list()
    for module in modules:
        if get_package(module) in BUILTIN:
            builtin.append(module)
        elif get_package(module) in third_party_packages:
            third_party.append(module)
        elif get_package(module) in homemade_packages:
            homemade.append(module)
        elif module.startswith('.'):
            homemade.append(module)
        else:
            raise ValueError(
                'Package {package} is not registered.'.format(
                    package=get_package(module),
                ),
            )

    # Create lines of import in the correct orders
    import_lines = list()
    for module_list in [builtin, third_party, homemade]:
        if module_list:
            if import_lines:
                import_lines.append('')
            for module in sorted(module_list):
                if module in import_imports:
                    import_lines.append('import ' + module)
                if module in from_imports:
                    import_line = 'from ' + module + ' import'
                    for idx, obj in enumerate(sorted(from_imports[module])):
                        if idx == len(from_imports[module]) - 1:
                            if len(import_line) + len(obj) < 79:
                                import_lines.append(import_line + ' ' + obj)
                            else:
                                import_lines.append(import_line + ' \\')
                                import_lines.append('    ' + obj)
                        else:
                            if len(import_line) + len(obj) < 76:
                                import_line += ' ' + obj + ','
                            else:
                                import_lines.append(import_line + ' \\')
                                import_line = '    ' + obj + ','

    # Join lines in a string and return
    content = '\n'.join(import_lines)
    if python_code:
        content += '\n\n\n' + python_code
    else:
        content += '\n'

    return content


def main():
    try:
        third_party_packages = list()
        homemade_packages = list()

        for argument in sys.argv:
            if argument.startswith('third_party='):
                third_party_packages = argument[12:].split(',')
            elif argument.startswith('homemade='):
                homemade_packages = argument[9:].split(',')

        with open(sys.argv[1], 'r') as fp:
            content = fp.read()
        content = organize(content, third_party_packages=third_party_packages,
                           homemade_packages=homemade_packages)
        with open(sys.argv[1], 'w') as fp:
            fp.write(content)
    except:
        print(sys.argv[1])
        raise


if __name__ == '__main__':
    main()
